/*
* Copyright (c) 2021 Huawei Device Co., Ltd.
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

/**
 * The process is mainly used to obtain the relevant ID of the process, obtain and modify
 * the working directory of the process, exit and close the process.
 * @since 7
 * @sysCap SystemCapability.CCRuntime
 * @devices phone, tablet, tv, wearable, car
 * @import import url from '@ohos.process';
 */
 declare namespace process {
    /**
    * returns the digital user id of the process
    * @since 7
    * @sysCap SystemCapability.CCRuntime
    * @return return the digital user id of the process.
    */
    const uid: number;

    /**
    * return pid is The pid of the current process
    * @since 7
    * @sysCap SystemCapability.CCRuntime
    * @return return The pid of the current process.
    */
    const pid: number;

    type EventListener = (evt: Object) => void;
    /**
    * Return a child process object and spawns a new ChildProcess to run the command
    * @since 7
    * @sysCap SystemCapability.CCRuntime
    * @param command string of the shell commands executed by the child process.
    * @param options This is an object. The object contains three parameters. Timeout is the running time of the child
    * process, killSignal is the signal sent when the child process reaches timeout, and maxBuffer is the size of the
    * maximum buffer area for standard input and output.
    * @return Return a child process object.
    */
    function runCmd(command: string,
        options?: { timeout : number, killSignal : number | string, maxBuffer : number }): ChildProcess;

    /**
    * Register for an event
    * @since 7
    * @sysCap SystemCapability.CCRuntime
    * @param type Indicates the type of event registered.
    * @param listener Represents the registered event function
    */
    function on(type: string, listener: EventListener): void;

    /**
    * Remove registered event
    * @since 7
    * @sysCap SystemCapability.CCRuntime
    * @param type Remove the type of registered event.
    * @return Return removed result.
    */
    function off(type: string): boolean;

    /**
    * Process exit
    * @since 7
    * @sysCap SystemCapability.CCRuntime
    * @param code Process exit code.
    */
    function exit(code: number): void;

    /**
    * Change current  directory
    * @since 7
    * @sysCap SystemCapability.CCRuntime
    * @param dir The path you want to change.
    */
    function chdir(dir: string): void;

    /**
     * Return whether the signal was sent successfully
     * @since 7
     * @sysCap SystemCapability.CCRuntime
     * @param signal Signal sent.
     * @param pid Send signal to target pid.
     * @return Return the result of the signal.
     */
    function kill(signal: number, pid: number): boolean;
}
export default process;
