import * as ts from "typescript";
export declare class DiagnosticError {
    irnode: ts.Node | undefined;
    code: number;
    file: ts.Node | undefined;
    args: (string | number | undefined)[];
    constructor(irnode: ts.Node | undefined, code: number, file?: ts.Node | undefined, args?: (string | number | undefined)[]);
}
export declare function printDiagnostic(diagnostic: ts.Diagnostic): void;
export declare function createDiagnosticOnFirstToken(file: ts.Node | undefined, node: ts.Node, message: ts.DiagnosticMessage | ts.DiagnosticMessageChain, ...args: (string | number | undefined)[]): any;
export declare function createFileDiagnostic(file: ts.Node | undefined, node: ts.Node, message: ts.DiagnosticMessage | ts.DiagnosticMessageChain, ...args: (string | number | undefined)[]): any;
export declare function createDiagnostic(file: ts.Node | undefined, location: ts.Node | undefined, message: ts.DiagnosticMessage | ts.DiagnosticMessageChain, ...args: (string | number | undefined)[]): any;
export declare enum DiagnosticCode {
    Identifier_expected = 1003,
    A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma = 1013,
    A_rest_parameter_must_be_last_in_a_parameter_list = 1014,
    Parameter_cannot_have_question_mark_and_initializer = 1015,
    A_required_parameter_cannot_follow_an_optional_parameter = 1016,
    The_readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature = 1024,
    Accessibility_modifier_already_seen = 1028,
    _0_modifier_must_precede_1_modifier = 1029,
    _0_modifier_already_seen = 1030,
    _0_modifier_cannot_appear_on_class_elements_of_this_kind = 1031,
    A_declare_modifier_cannot_be_used_in_an_already_ambient_context = 1038,
    _0_modifier_cannot_be_used_in_an_ambient_context = 1040,
    _0_modifier_cannot_be_used_here = 1042,
    _0_modifier_cannot_appear_on_a_module_or_namespace_element = 1044,
    A_rest_parameter_cannot_be_optional = 1047,
    A_rest_parameter_cannot_have_an_initializer = 1048,
    _0_modifier_cannot_appear_on_a_type_member = 1070,
    _0_modifier_cannot_appear_on_an_index_signature = 1071,
    A_0_modifier_cannot_be_used_with_an_import_declaration = 1079,
    _0_modifier_cannot_appear_on_a_constructor_declaration = 1089,
    _0_modifier_cannot_appear_on_a_parameter = 1090,
    Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement = 1091,
    Invalid_use_of_0_in_strict_mode = 1100,
    A_with_statements_are_not_allowed_in_strict_mode = 1101,
    A_delete_cannot_be_called_on_an_identifier_in_strict_mode = 1102,
    A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement = 1104,
    A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement = 1105,
    Jump_target_cannot_cross_function_boundary = 1107,
    A_return_statement_can_only_be_used_within_a_function_body = 1108,
    A_default_clause_cannot_appear_more_than_once_in_a_switch_statement = 1113,
    Duplicate_label_0 = 1114,
    A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement = 1115,
    A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement = 1116,
    An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode = 1117,
    An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name = 1118,
    An_object_literal_cannot_have_property_and_accessor_with_the_same_name = 1119,
    Octal_literals_are_not_allowed_in_strict_mode = 1121,
    Octal_escape_sequences_are_not_allowed_in_strict_mode = 1122,
    Variable_declaration_list_cannot_be_empty = 1123,
    Line_break_not_permitted_here = 1142,
    The_const_declarations_can_only_be_declared_inside_a_block = 1156,
    The_const_declarations_must_be_initialized = 1155,
    The_let_declarations_can_only_be_declared_inside_a_block = 1157,
    Unterminated_regular_expression_literal = 1161,
    An_object_member_cannot_be_declared_optional = 1162,
    A_yield_expression_is_only_allowed_in_a_generator_body = 1163,
    A_comma_expression_is_not_allowed_in_a_computed_property_name = 1171,
    The_extends_clause_already_seen = 1172,
    Classes_can_only_extend_a_single_class = 1174,
    The_implements_clause_already_seen = 1175,
    Property_destructuring_pattern_expected = 1180,
    A_destructuring_declaration_must_have_an_initializer = 1182,
    A_rest_element_cannot_have_an_initializer = 1186,
    A_parameter_property_may_not_be_declared_using_a_binding_pattern = 1187,
    Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement = 1188,
    The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer = 1189,
    The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer = 1190,
    Line_terminator_not_permitted_before_arrow = 1200,
    Decorators_are_not_valid_here = 1206,
    Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name = 1207,
    Invalid_use_of_0_Class_definitions_are_automatically_in_strict_mode = 1210,
    Identifier_expected_0_is_a_reserved_word_in_strict_mode = 1212,
    Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_strict_mode = 1213,
    Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode = 1214,
    An_import_declaration_can_only_be_used_in_a_namespace_or_module = 1232,
    An_export_declaration_can_only_be_used_in_a_module = 1233,
    The_abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration = 1242,
    _0_modifier_cannot_be_used_with_1_modifier = 1243,
    Abstract_methods_can_only_appear_within_an_abstract_class = 1244,
    A_class_member_cannot_have_the_0_keyword = 1248,
    A_decorator_can_only_decorate_a_method_implementation_not_an_overload = 1249,
    Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5 = 1250,
    Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_definitions_are_automatically_in_strict_mode = 1251,
    Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_are_automatically_in_strict_mode = 1252,
    A_definite_assignment_assertion_is_not_permitted_in_this_context = 1255,
    Identifier_expected_0_is_a_reserved_word_at_the_top_level_of_a_module = 1262,
    Declarations_with_initializers_cannot_also_have_definite_assignment_assertions = 1263,
    Declarations_with_definite_assignment_assertions_must_also_have_type_annotations = 1264,
    await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules = 1308,
    Did_you_mean_to_use_a_Colon_An_can_only_follow_a_property_name_when_the_containing_object_literal_is_part_of_a_destructuring_pattern = 1312,
    A_parameter_property_cannot_be_declared_using_a_rest_parameter = 1317,
    A_default_export_can_only_be_used_in_an_ECMAScript_style_module = 1319,
    use_strict_directive_cannot_be_used_with_non_simple_parameter_list = 1347,
    Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here = 1359,
    Duplicate_identifier_0 = 2300,
    The_super_can_only_be_referenced_in_a_derived_class = 2335,
    The_super_cannot_be_referenced_in_constructor_arguments = 2336,
    Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors = 2337,
    The_super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_derived_class = 2338,
    The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_parameter = 2358,
    The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_Function_interface_type = 2359,
    The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol = 2360,
    The_right_hand_side_of_an_in_expression_must_be_of_type_any_an_object_type_or_a_type_parameter = 2361,
    The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type = 2362,
    The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type = 2363,
    The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access = 2364,
    Multiple_constructor_implementations_are_not_allowed = 2392,
    Declaration_name_conflicts_with_built_in_global_identifier_0 = 2397,
    The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation = 2404,
    The_super_cannot_be_referenced_in_a_computed_property_name = 2466,
    A_rest_element_must_be_last_in_a_destructuring_pattern = 2462,
    The_let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations = 2480,
    The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation = 2483,
    The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access = 2487,
    The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern = 2491,
    A_rest_element_cannot_contain_a_binding_pattern = 2501,
    The_super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions = 2660,
    Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module = 2661,
    Left_side_of_comma_operator_is_unused_and_has_no_side_effects = 2695,
    The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access = 2701,
    The_target_of_an_object_rest_assignment_may_not_be_an_optional_property_access = 2778,
    The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access = 2779,
    The_left_hand_side_of_a_for_of_statement_may_not_be_an_optional_property_access = 2781,
    _0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2 = 17012,
    Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constructor = 17013,
    An_accessibility_modifier_cannot_be_used_with_a_private_identifier = 18010,
    Private_identifiers_are_not_allowed_outside_class_bodies = 18016,
    _0_modifier_cannot_be_used_with_a_private_identifier = 18019,
    In_strict_mode_code_functions_can_only_be_declared_at_top_level_or_inside_a_block = 19000,
    Class_Declaration_can_only_be_declared_at_top_level_or_inside_a_block = 19001,
    Incorrect_regular_expression = 19002,
    Invalid_regular_expression_Colon_0_Colon_Invalid_escape = 19003,
    _8_and_9_are_not_allowed_in_strict_mode = 19004,
    const_and_let_declarations_not_allowed_in_statement_positions = 19005,
    Getter_must_not_have_any_formal_parameters = 19006,
    Class_declaration_not_allowed_in_statement_position = 19007,
    Lexical_declaration_let_not_allowed_in_statement_position = 19008,
    Lexical_declaration_const_not_allowed_in_statement_position = 19009,
    Invalid_regular_expression_flag_0 = 19010
}
export declare function getDiagnostic(code: DiagnosticCode): ts.DiagnosticMessage | undefined;
//# sourceMappingURL=diagnostic.d.ts.map